/***************************************************************************
                          klines.cpp  -  description
                             -------------------
    begin                : Fri May 19 2000
    copyright            : (C) 2000 by Roman Merzlyakov
    email                : roman@sbrf.barrt.ru
    copyright            : (C) 2000 by Roman Razilov
    email                : Roman.Razilov@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* changes
21.05.2000    Roman Razilov     Menu game/Next
*/
//
// The implementation of the KLines widget
//

#include <qpopmenu.h>
#include <qkeycode.h>
#include <qlabel.h>
#include <qpushbt.h>
#include <qtooltip.h>
#include <qstring.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#include <kapp.h>
#include <kiconloader.h>
#include <klocale.h>
#define trans klocale

#include "cfg.h"
#include "klines.h"

/*
   Creates the KLines widget and sets saved options (if any).
*/

KLines::KLines() : KTopLevelWidget()
{
  time_t t;
  time(&t);
  srand((unsigned int)t + getpid());

  setCaption(QString("KLines ")+LINESVERSION);

  hs = new HScore();

  mwidget = new MainWidget(this);
//  connect(lsb, SIGNAL(endTurn()), this, SLOT(makeTurn()));
//  connect(lsb, SIGNAL(eraseLine(int)), this, SLOT(addScore(int)));

  lsb = mwidget->GetLsb();
  lPrompt = mwidget->GetPrompt();
  
  menu              = new KMenuBar(this, "menu");
  game              = new QPopupMenu;
  QPopupMenu * file = new QPopupMenu;
  QPopupMenu * help = new QPopupMenu;
  options = new QPopupMenu;

  CHECK_PTR( file );
  CHECK_PTR( game );
  CHECK_PTR( help );
  CHECK_PTR( menu );
  CHECK_PTR( options );

  help->insertItem( trans->translate("&Help"), this, SLOT(help()), CTRL+Key_H );
  
  file->insertItem( trans->translate("&Quit"), kapp, SLOT(quit()), CTRL+Key_Q );
  game->insertItem( "Next", this, SLOT(makeTurn()), CTRL+Key_N );
  game->insertItem( "Restart game", this, SLOT(startGame()), CTRL+Key_R );
  game->insertItem( "High score", hs, SLOT(viewHighScore()), CTRL+Key_H );
  
  options->setCheckable(true);
  idMenuPrompt = options->insertItem( "Show next balls", this, SLOT(switchPrompt()) );
  idMenuAnimateMoving = options->insertItem( "Animate moving", this, SLOT(switchAnimateMoving()) );
  
  menu->insertItem( trans->translate("&File"), file );
  menu->insertItem( trans->translate("&Game"), game );
  menu->insertItem( trans->translate("&Options"), options );
  menu->insertSeparator();
  menu->insertItem( trans->translate("&Help"), help );

  menu->show();
  setMenu( menu );

  score = 0;
  prev_score = 0;

  QString s;
  stat = new KStatusBar(this);
  s = "score: xxxx";
  stat->insertItem((const char*)s, LSCORE);
  s = "record: xxxx";
  stat->insertItem((const char*)s, LRECORD);
  setStatusBar(stat);

  setView(mwidget);
  
  ReadConfig();
  
  startGame();
}

/*
   Saves the options and destroys the KLines widget.
*/

KLines::~KLines()
{
    WriteConfig();
}

/*
   Resize event of the KLines widget.
*/

void KLines::resizeEvent( QResizeEvent *e )
{
  KTopLevelWidget::resizeEvent(e);
}

void KLines::setMinSize()
{
//  setMinimumSize( gr->wHint(), gr->hHint() + menu->height() + stat->height() +
//      tool->height() );
}

void KLines::startGame()
{
    score = 0;
    prev_score = 0;

    lsb->clearField();
    generateRandomBalls();
    placeBalls();
    generateRandomBalls();

    updateStat();
}

void KLines::stopGame()
{
  debug("KLines::stopGame");
  endGame();
}

void KLines::searchBallsLine()
{
}

void KLines::generateRandomBalls()
{
    nextBalls[0] = rand() % NCOLORS;
    nextBalls[1] = rand() % NCOLORS;
    nextBalls[2] = rand() % NCOLORS;
    lPrompt->SetBalls(nextBalls[0], nextBalls[1], nextBalls[2]);
}

void KLines::placeBalls()
{
    lsb->placeBalls(nextBalls);
    debug("exit from placeBalls");
}

void KLines::makeTurn()
{

    placeBalls();
    generateRandomBalls();

}

void KLines::addScore(int ballsErased)
{   if(ballsErased >= 5){
      score += 2*ballsErased*ballsErased - 20*ballsErased + 60 ;
      if( !lPrompt->getState() ) score+= 1;
      updateStat();
    };
}

void KLines::updateStat()
{
    QString s;
    s.sprintf("score: %i", score);
    stat->changeItem((const char*)s, LSCORE);
    s.sprintf("record: %i", hs->getMaxScore());
    stat->changeItem((const char*)s, LRECORD);
}

void KLines::endGame()
{
    if(score > hs->getMaxScore() ) updateStat();
    hs->slotEndOfGame(score);

    startGame();
}

void KLines::switchPrompt()
{
    lPrompt->setPromptAndApply(!lPrompt->getState());
    options->setItemChecked(idMenuPrompt, lPrompt->getState());
}

void KLines::help()
{
  KApplication::getKApplication()->invokeHTMLHelp("", "");
}

void KLines::ReadConfig()
{
  QString s, e, grp;
  KConfig *conf = kapp->getConfig();
  grp = conf->group();
  conf->setGroup("Options");
  
  s.sprintf("AnimateMoving");
  if(conf->hasKey(s)) {
    e = conf->readEntry(s.data());
    if(e.toInt()==0) lsb->setAnimateMoving(false);
    else lsb->setAnimateMoving(true);
  };
  
  s.sprintf("ShowPrompt");
  if(conf->hasKey(s)) {
    e = conf->readEntry(s.data());
    if(e.toInt()==0) lPrompt->setPrompt(false); 
    else lPrompt->setPrompt(true); 
  };
  
  options->setItemChecked(idMenuAnimateMoving, lsb->getAnimateMoving());
  options->setItemChecked(idMenuPrompt, lPrompt->getState());
  
  // restore old group
  conf->setGroup(grp.data());

}

void KLines::WriteConfig()
{
  QString s, e, grp;
  
  KConfig *conf = kapp->getConfig();
  
  grp = conf->group();
  conf->setGroup("Options");
    
  s.sprintf("AnimateMoving");
  if(lsb->getAnimateMoving()) e="1";
  else e="0";
  conf->writeEntry(s, e);
  
  s.sprintf("ShowPrompt");
  if(lPrompt->getState()) e="1";
  else e="0";
  conf->writeEntry(s, e);
  
  // restore old group
  conf->setGroup(grp.data());
 
}

void KLines::switchAnimateMoving(){
    if(lsb->getAnimateMoving()) lsb->setAnimateMoving(false);
    else lsb->setAnimateMoving(true);
    options->setItemChecked(idMenuAnimateMoving, lsb->getAnimateMoving());
}